from _typeshed import Incomplete
from typing import ClassVar, Literal

from openpyxl.descriptors.base import String
from openpyxl.descriptors.serialisable import Serialisable

class CellWatch(Serialisable):
    tagname: ClassVar[str]
    r: String[Literal[True]]
    def __init__(self, r: str) -> None: ...

class CellWatches(Serialisable):
    tagname: ClassVar[str]
    cellWatch: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, cellWatch=()) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from openpyxl.descriptors.base import Bool, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

class Extension(Serialisable):
    tagname: ClassVar[str]
    uri: String[Literal[True]]
    def __init__(self, uri: str | None = None) -> None: ...

class ExtensionList(Serialisable):
    tagname: ClassVar[str]
    ext: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, ext=()) -> None: ...

class IgnoredError(Serialisable):
    tagname: ClassVar[str]
    sqref: Incomplete
    evalError: Bool[Literal[True]]
    twoDigitTextYear: Bool[Literal[True]]
    numberStoredAsText: Bool[Literal[True]]
    formula: Bool[Literal[True]]
    formulaRange: Bool[Literal[True]]
    unlockedFormula: Bool[Literal[True]]
    emptyCellReference: Bool[Literal[True]]
    listDataValidation: Bool[Literal[True]]
    calculatedColumn: Bool[Literal[True]]
    def __init__(
        self,
        sqref=None,
        evalError: _ConvertibleToBool | None = False,
        twoDigitTextYear: _ConvertibleToBool | None = False,
        numberStoredAsText: _ConvertibleToBool | None = False,
        formula: _ConvertibleToBool | None = False,
        formulaRange: _ConvertibleToBool | None = False,
        unlockedFormula: _ConvertibleToBool | None = False,
        emptyCellReference: _ConvertibleToBool | None = False,
        listDataValidation: _ConvertibleToBool | None = False,
        calculatedColumn: _ConvertibleToBool | None = False,
    ) -> None: ...

class IgnoredErrors(Serialisable):
    tagname: ClassVar[str]
    ignoredError: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, ignoredError=(), extLst: ExtensionList | None = None) -> None: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete, Unused
from datetime import datetime
from typing import ClassVar, Final, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import (
    Alias,
    Bool,
    DateTime,
    Float,
    Integer,
    MinMax,
    NoneSet,
    Set,
    String,
    Typed,
    _ConvertibleToBool,
)
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

_SortConditionSortBy: TypeAlias = Literal["value", "cellColor", "fontColor", "icon"]
_IconSet: TypeAlias = Literal[
    "3Arrows",
    "3ArrowsGray",
    "3Flags",
    "3TrafficLights1",
    "3TrafficLights2",
    "3Signs",
    "3Symbols",
    "3Symbols2",
    "4Arrows",
    "4ArrowsGray",
    "4RedToBlack",
    "4Rating",
    "4TrafficLights",
    "5Arrows",
    "5ArrowsGray",
    "5Rating",
    "5Quarters",
]
_SortStateSortMethod: TypeAlias = Literal["stroke", "pinYin"]
_CustomFilterOperator: TypeAlias = Literal[
    "equal", "lessThan", "lessThanOrEqual", "notEqual", "greaterThanOrEqual", "greaterThan"
]
_StringFilterOperator: TypeAlias = Literal["contains", "startswith", "endswith", "wildcard"]
_FiltersCalendarType: TypeAlias = Literal[
    "gregorian",
    "gregorianUs",
    "gregorianMeFrench",
    "gregorianArabic",
    "hijri",
    "hebrew",
    "taiwan",
    "japan",
    "thai",
    "korea",
    "saka",
    "gregorianXlitEnglish",
    "gregorianXlitFrench",
]
_DynamicFilterType: TypeAlias = Literal[
    "null",
    "aboveAverage",
    "belowAverage",
    "tomorrow",
    "today",
    "yesterday",
    "nextWeek",
    "thisWeek",
    "lastWeek",
    "nextMonth",
    "thisMonth",
    "lastMonth",
    "nextQuarter",
    "thisQuarter",
    "lastQuarter",
    "nextYear",
    "thisYear",
    "lastYear",
    "yearToDate",
    "Q1",
    "Q2",
    "Q3",
    "Q4",
    "M1",
    "M2",
    "M3",
    "M4",
    "M5",
    "M6",
    "M7",
    "M8",
    "M9",
    "M10",
    "M11",
    "M12",
]
_DateGroupItemDateTimeGrouping: TypeAlias = Literal["year", "month", "day", "hour", "minute", "second"]

class SortCondition(Serialisable):
    tagname: ClassVar[str]
    descending: Bool[Literal[True]]
    sortBy: NoneSet[_SortConditionSortBy]
    ref: Incomplete
    customList: String[Literal[True]]
    dxfId: Integer[Literal[True]]
    iconSet: NoneSet[_IconSet]
    iconId: Integer[Literal[True]]
    def __init__(
        self,
        ref=None,
        descending: _ConvertibleToBool | None = None,
        sortBy: _SortConditionSortBy | Literal["none"] | None = None,
        customList: str | None = None,
        dxfId: ConvertibleToInt | None = None,
        iconSet: _IconSet | Literal["none"] | None = None,
        iconId: ConvertibleToInt | None = None,
    ) -> None: ...

class SortState(Serialisable):
    tagname: ClassVar[str]
    columnSort: Bool[Literal[True]]
    caseSensitive: Bool[Literal[True]]
    sortMethod: NoneSet[_SortStateSortMethod]
    ref: Incomplete
    sortCondition: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        columnSort: _ConvertibleToBool | None = None,
        caseSensitive: _ConvertibleToBool | None = None,
        sortMethod: _SortStateSortMethod | Literal["none"] | None = None,
        ref=None,
        sortCondition=(),
        extLst: Unused = None,
    ) -> None: ...
    def __bool__(self) -> bool: ...

class IconFilter(Serialisable):
    tagname: ClassVar[str]
    iconSet: Set[_IconSet]
    iconId: Integer[Literal[True]]
    def __init__(self, iconSet: _IconSet, iconId: ConvertibleToInt | None = None) -> None: ...

class ColorFilter(Serialisable):
    tagname: ClassVar[str]
    dxfId: Integer[Literal[True]]
    cellColor: Bool[Literal[True]]
    def __init__(self, dxfId: ConvertibleToInt | None = None, cellColor: _ConvertibleToBool | None = None) -> None: ...

class DynamicFilter(Serialisable):
    tagname: ClassVar[str]
    type: Set[_DynamicFilterType]
    val: Float[Literal[True]]
    valIso: DateTime[Literal[True]]
    maxVal: Float[Literal[True]]
    maxValIso: DateTime[Literal[True]]
    def __init__(
        self,
        type: _DynamicFilterType,
        val: ConvertibleToFloat | None = None,
        valIso: datetime | str | None = None,
        maxVal: ConvertibleToFloat | None = None,
        maxValIso: datetime | str | None = None,
    ) -> None: ...

class CustomFilter(Serialisable):
    tagname: ClassVar[str]
    val: String[Literal[False]]
    operator: Set[_CustomFilterOperator]
    def __init__(self, operator: _CustomFilterOperator = "equal", val: str | None = None) -> None: ...
    def convert(self) -> BlankFilter | NumberFilter | StringFilter: ...

class BlankFilter(CustomFilter):
    def __init__(self, **kw: Unused) -> None: ...
    @property
    def operator(self) -> Literal["notEqual"]: ...  # type: ignore[override]
    @property
    def val(self) -> Literal[" "]: ...  # type: ignore[override]

class NumberFilter(CustomFilter):
    val: Float[Literal[False]]  # type: ignore[assignment]
    def __init__(self, operator: _CustomFilterOperator = "equal", val: ConvertibleToFloat | None = None) -> None: ...

string_format_mapping: Final[dict[_StringFilterOperator, str]]

class StringFilter(CustomFilter):
    operator: Set[_StringFilterOperator]  # type: ignore[assignment]
    val: String[Literal[False]]
    exclude: Bool[Literal[False]]
    def __init__(
        self, operator: _StringFilterOperator = "contains", val: str | None = None, exclude: _ConvertibleToBool = False
    ) -> None: ...

class CustomFilters(Serialisable):
    tagname: ClassVar[str]
    _and: Bool[Literal[True]]  # Not private. Avoids name clash
    customFilter: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, _and: _ConvertibleToBool | None = None, customFilter=()) -> None: ...

class Top10(Serialisable):
    tagname: ClassVar[str]
    top: Bool[Literal[True]]
    percent: Bool[Literal[True]]
    val: Float[Literal[False]]
    filterVal: Float[Literal[True]]
    @overload
    def __init__(
        self,
        top: _ConvertibleToBool | None = None,
        percent: _ConvertibleToBool | None = None,
        *,
        val: ConvertibleToFloat,
        filterVal: ConvertibleToFloat | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        top: _ConvertibleToBool | None,
        percent: _ConvertibleToBool | None,
        val: ConvertibleToFloat,
        filterVal: ConvertibleToFloat | None = None,
    ) -> None: ...

class DateGroupItem(Serialisable):
    tagname: ClassVar[str]
    year: Integer[Literal[False]]
    month: MinMax[float, Literal[True]]
    day: MinMax[float, Literal[True]]
    hour: MinMax[float, Literal[True]]
    minute: MinMax[float, Literal[True]]
    second: Integer[Literal[True]]
    dateTimeGrouping: Set[_DateGroupItemDateTimeGrouping]
    @overload
    def __init__(
        self,
        year: ConvertibleToInt,
        month: ConvertibleToFloat | None = None,
        day: ConvertibleToFloat | None = None,
        hour: ConvertibleToFloat | None = None,
        minute: ConvertibleToFloat | None = None,
        second: ConvertibleToInt | None = None,
        *,
        dateTimeGrouping: _DateGroupItemDateTimeGrouping,
    ) -> None: ...
    @overload
    def __init__(
        self,
        year: ConvertibleToInt,
        month: ConvertibleToFloat | None,
        day: ConvertibleToFloat | None,
        hour: ConvertibleToFloat | None,
        minute: ConvertibleToFloat | None,
        second: ConvertibleToInt | None,
        dateTimeGrouping: _DateGroupItemDateTimeGrouping,
    ) -> None: ...

class Filters(Serialisable):
    tagname: ClassVar[str]
    blank: Bool[Literal[True]]
    calendarType: NoneSet[_FiltersCalendarType]
    filter: Incomplete
    dateGroupItem: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        blank: _ConvertibleToBool | None = None,
        calendarType: _FiltersCalendarType | Literal["none"] | None = None,
        filter=(),
        dateGroupItem=(),
    ) -> None: ...

class FilterColumn(Serialisable):
    tagname: ClassVar[str]
    colId: Integer[Literal[False]]
    col_id: Alias
    hiddenButton: Bool[Literal[True]]
    showButton: Bool[Literal[True]]
    filters: Typed[Filters, Literal[True]]
    top10: Typed[Top10, Literal[True]]
    customFilters: Typed[CustomFilters, Literal[True]]
    dynamicFilter: Typed[DynamicFilter, Literal[True]]
    colorFilter: Typed[ColorFilter, Literal[True]]
    iconFilter: Typed[IconFilter, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        colId: ConvertibleToInt,
        hiddenButton: _ConvertibleToBool | None = False,
        showButton: _ConvertibleToBool | None = True,
        filters: Filters | None = None,
        top10: Top10 | None = None,
        customFilters: CustomFilters | None = None,
        dynamicFilter: DynamicFilter | None = None,
        colorFilter: ColorFilter | None = None,
        iconFilter: IconFilter | None = None,
        extLst: Unused = None,
        blank=None,
        vals=None,
    ) -> None: ...

class AutoFilter(Serialisable):
    tagname: ClassVar[str]
    ref: Incomplete
    filterColumn: Incomplete
    sortState: Typed[SortState, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, ref=None, filterColumn=(), sortState: SortState | None = None, extLst: Unused = None) -> None: ...
    def __bool__(self) -> bool: ...
    def add_filter_column(self, col_id, vals, blank: bool = False) -> None: ...
    def add_sort_condition(self, ref, descending: bool = False) -> None: ...



from _typeshed import Unused

from openpyxl.cell import _CellOrMergedCell

def etree_write_cell(xf, worksheet: Unused, cell: _CellOrMergedCell, styled=None) -> None: ...
def lxml_write_cell(xf, worksheet: Unused, cell: _CellOrMergedCell, styled: bool = False) -> None: ...

write_cell = lxml_write_cell
write_cell = etree_write_cell



from typing import Final, Literal
from typing_extensions import TypeAlias
from zipfile import ZipFile

from openpyxl import _ZipFileFileProtocol
from openpyxl.chartsheet.chartsheet import Chartsheet
from openpyxl.packaging.manifest import Manifest
from openpyxl.packaging.relationship import Relationship
from openpyxl.reader.workbook import WorkbookParser
from openpyxl.workbook import Workbook

_SupportedFormats: TypeAlias = Literal[".xlsx", ".xlsm", ".xltx", ".xltm"]
SUPPORTED_FORMATS: Final[tuple[_SupportedFormats, ...]]

class ExcelReader:
    archive: ZipFile
    valid_files: list[str]
    read_only: bool
    keep_vba: bool
    data_only: bool
    keep_links: bool
    rich_text: bool
    shared_strings: list[str]
    package: Manifest  # defined after call to read_manifest()
    parser: WorkbookParser  # defined after call to read_workbook()
    wb: Workbook  # defined after call to read_workbook()

    def __init__(
        self,
        fn: _ZipFileFileProtocol,
        read_only: bool = False,
        keep_vba: bool = False,
        data_only: bool = False,
        keep_links: bool = True,
        rich_text: bool = False,
    ) -> None: ...
    def read_manifest(self) -> None: ...
    def read_strings(self) -> None: ...
    def read_workbook(self) -> None: ...
    def read_properties(self) -> None: ...
    def read_custom(self) -> None: ...
    def read_theme(self) -> None: ...
    def read_chartsheet(self, sheet: Chartsheet, rel: Relationship) -> None: ...
    def read_worksheets(self) -> None: ...
    def read(self) -> None: ...

def load_workbook(
    filename: _ZipFileFileProtocol,
    read_only: bool = False,
    keep_vba: bool = False,
    data_only: bool = False,
    keep_links: bool = True,
    rich_text: bool = False,
) -> Workbook: ...



from _typeshed import Incomplete
from typing import Final

from openpyxl.cell import _CellGetValue
from openpyxl.styles.alignment import Alignment
from openpyxl.styles.borders import Border
from openpyxl.styles.cell_style import StyleArray
from openpyxl.styles.fills import Fill
from openpyxl.styles.fonts import Font
from openpyxl.styles.protection import Protection
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.worksheet._read_only import ReadOnlyWorksheet

class ReadOnlyCell:
    __slots__ = ("parent", "row", "column", "_value", "data_type", "_style_id")
    parent: _WorkbookChild | ReadOnlyWorksheet
    row: Incomplete
    column: Incomplete
    data_type: Incomplete
    def __init__(
        self, sheet: _WorkbookChild | ReadOnlyWorksheet, row, column, value, data_type: str = "n", style_id: int = 0
    ) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    # Same as Cell.coordinate
    # https://github.com/python/mypy/issues/6700
    # Defined twice in the implementation
    @property
    def coordinate(self) -> str: ...
    # Same as Cell.column_letter
    # https://github.com/python/mypy/issues/6700
    @property
    def column_letter(self) -> str: ...
    @property
    def style_array(self) -> StyleArray: ...
    @property
    def has_style(self) -> bool: ...
    @property
    def number_format(self) -> str: ...
    @property
    def font(self) -> Font: ...
    @property
    def fill(self) -> Fill: ...
    @property
    def border(self) -> Border: ...
    @property
    def alignment(self) -> Alignment: ...
    @property
    def protection(self) -> Protection: ...
    # Same as Cell.is_date
    # https://github.com/python/mypy/issues/6700
    @property
    def is_date(self) -> bool: ...
    @property
    def internal_value(self) -> _CellGetValue: ...
    @property
    def value(self) -> _CellGetValue: ...
    @value.setter
    def value(self, value: None) -> None: ...

class EmptyCell:
    __slots__ = ()
    value: Incomplete
    is_date: bool
    font: Incomplete
    border: Incomplete
    fill: Incomplete
    number_format: Incomplete
    alignment: Incomplete
    data_type: str

EMPTY_CELL: Final[EmptyCell]



from _typeshed import Incomplete, Unused
from collections.abc import Generator, Iterable, Sized
from typing import Any, Protocol, TypeVar, type_check_only
from typing_extensions import Self

from openpyxl.descriptors import Strict
from openpyxl.descriptors.serialisable import Serialisable, _SerialisableTreeElement
from openpyxl.xml._functions_overloads import _HasGet
from openpyxl.xml.functions import Element

from .base import Alias, Descriptor

_T = TypeVar("_T")
_ContainerT = TypeVar("_ContainerT")

@type_check_only
class _SupportsFromTree(Protocol):
    @classmethod
    def from_tree(cls, node: _SerialisableTreeElement) -> Any: ...

@type_check_only
class _SupportsToTree(Protocol):
    def to_tree(self) -> Element: ...

# `_ContainerT` is the internal container type (which defaults to `list`), or
# `IndexedList` if unique is `True`.
class Sequence(Descriptor[_ContainerT]):
    expected_type: type[Any]  # expected type of the sequence elements
    seq_types: tuple[type, ...]  # allowed settable sequence types, defaults to `list`, `tuple`
    idx_base: int
    unique: bool
    container: type  # internal container type, defaults to `list`
    # seq must be an instance of any of the declared `seq_types`.
    def __set__(self, instance: Serialisable | Strict, seq: Any) -> None: ...
    def to_tree(
        self, tagname: str | None, obj: Iterable[object], namespace: str | None = None
    ) -> Generator[Element, None, None]: ...

# `_T` is the type of the elements in the sequence.
class UniqueSequence(Sequence[set[_T]]):
    seq_types: tuple[type[list[_T]], type[tuple[_T, ...]], type[set[_T]]]
    container: type[set[_T]]

# See `Sequence` for the meaning of `_ContainerT`.
class ValueSequence(Sequence[_ContainerT]):
    attribute: str
    def to_tree(
        self, tagname: str, obj: Iterable[object], namespace: str | None = None  # type: ignore[override]
    ) -> Generator[Element, None, None]: ...
    def from_tree(self, node: _HasGet[_T]) -> _T: ...

@type_check_only
class _NestedSequenceToTreeObj(Sized, Iterable[_SupportsToTree], Protocol): ...

# See `Sequence` for the meaning of `_ContainerT`.
class NestedSequence(Sequence[_ContainerT]):
    count: bool
    expected_type: type[_SupportsFromTree]
    def to_tree(  # type: ignore[override]
        self, tagname: str, obj: _NestedSequenceToTreeObj, namespace: str | None = None
    ) -> Element: ...
    # returned list generic type should be same as the return type of expected_type.from_tree(node)
    # Which can really be anything given the wildly different, and sometimes generic, from_tree return types
    def from_tree(self, node: Iterable[_SerialisableTreeElement]) -> list[Any]: ...

# `_T` is the type of the elements in the sequence.
class MultiSequence(Sequence[list[_T]]):
    def __set__(self, instance: Serialisable | Strict, seq: tuple[_T, ...] | list[_T]) -> None: ...
    def to_tree(
        self, tagname: Unused, obj: Iterable[_SupportsToTree], namespace: str | None = None  # type: ignore[override]
    ) -> Generator[Element, None, None]: ...

class MultiSequencePart(Alias):
    expected_type: type[Incomplete]
    store: Incomplete
    def __init__(self, expected_type, store) -> None: ...
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...
    def __get__(self, instance: Unused, cls: Unused) -> Self: ...



from datetime import date, datetime, time, timedelta
from decimal import Decimal
from typing import Any
from typing_extensions import TypeAlias

from openpyxl.cell.rich_text import CellRichText
from openpyxl.worksheet.formula import ArrayFormula, DataTableFormula

from .cell import Cell as Cell, MergedCell as MergedCell, WriteOnlyCell as WriteOnlyCell
from .read_only import ReadOnlyCell as ReadOnlyCell

_TimeTypes: TypeAlias = datetime | date | time | timedelta
_CellGetValue: TypeAlias = (  # noqa: Y047 # Used in other modules
    # if numpy is installed also numpy bool and number types
    bool
    | float
    | Decimal
    | str
    | CellRichText
    | _TimeTypes
    | DataTableFormula
    | ArrayFormula
    | None
)
_AnyCellValue: TypeAlias = Any  # AnyOf _CellGetValue # noqa: Y047 # Used in other modules
_CellSetValue: TypeAlias = _CellGetValue | bytes  # noqa: Y047 # Used in other modules

_CellOrMergedCell: TypeAlias = Cell | MergedCell  # noqa: Y047 # Used in other modules



from _typeshed import SupportsRead
from pathlib import Path
from types import ModuleType
from typing import Any, Literal
from typing_extensions import TypeAlias

from openpyxl.drawing.spreadsheet_drawing import _AnchorBase

# Is actually PIL.Image.Image
_PILImageImage: TypeAlias = Any
# same as first parameter of PIL.Image.open
_PILImageFilePath: TypeAlias = str | bytes | Path | SupportsRead[bytes]

PILImage: ModuleType | Literal[False]

class Image:
    anchor: str | _AnchorBase
    ref: _PILImageImage | _PILImageFilePath
    width: int
    height: int
    format: str
    def __init__(self, img: _PILImageImage | _PILImageFilePath) -> None: ...
    @property
    def path(self) -> str: ...



from _typeshed import Incomplete, Unused
from collections.abc import Generator
from typing import ClassVar, Literal, TypeVar, overload
from zipfile import ZipFile

from openpyxl.descriptors.base import Alias, String
from openpyxl.descriptors.container import ElementList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.pivot.cache import CacheDefinition
from openpyxl.pivot.record import RecordList
from openpyxl.pivot.table import TableDefinition

_SerialisableT = TypeVar("_SerialisableT", bound=Serialisable)
_SerialisableRelTypeT = TypeVar("_SerialisableRelTypeT", bound=CacheDefinition | RecordList | TableDefinition)

class Relationship(Serialisable):
    tagname: ClassVar[str]
    Type: String[Literal[False]]
    Target: String[Literal[False]]
    target: Alias
    TargetMode: String[Literal[True]]
    Id: String[Literal[True]]
    id: Alias
    @overload
    def __init__(
        self, Id: str, Type: Unused = None, *, type: str, Target: str | None = None, TargetMode: str | None = None
    ) -> None: ...
    @overload
    def __init__(self, Id: str, Type: Unused, type: str, Target: str | None = None, TargetMode: str | None = None) -> None: ...
    @overload
    def __init__(
        self, Id: str, Type: str, type: None = None, Target: str | None = None, TargetMode: str | None = None
    ) -> None: ...

class RelationshipList(ElementList[Relationship]):
    expected_type: type[Relationship]
    def find(self, content_type: str) -> Generator[Relationship, None, None]: ...
    def get(self, key: str) -> Relationship: ...
    def to_dict(self) -> dict[Incomplete, Relationship]: ...

def get_rels_path(path): ...
def get_dependents(archive: ZipFile, filename: str) -> RelationshipList: ...

# If `id` is None, `cls` needs to have ClassVar `rel_type`.
# The `deps` attribute used at runtime is for internal use immediately after the return.
# `cls` cannot be None
@overload
def get_rel(
    archive: ZipFile, deps: RelationshipList, id: None = None, *, cls: type[_SerialisableRelTypeT]
) -> _SerialisableRelTypeT | None: ...
@overload
def get_rel(
    archive: ZipFile, deps: RelationshipList, id: None, cls: type[_SerialisableRelTypeT]
) -> _SerialisableRelTypeT | None: ...
@overload
def get_rel(archive: ZipFile, deps: RelationshipList, id: str, cls: type[_SerialisableT]) -> _SerialisableT: ...



from _typeshed import Incomplete, Self

from .base import *
from .sequence import Sequence as Sequence

class MetaStrict(type):
    def __new__(cls: type[Self], clsname: str, bases: tuple[type, ...], methods: dict[str, Descriptor[Incomplete]]) -> Self: ...

class MetaSerialisable(type):
    def __new__(cls: type[Self], clsname: str, bases: tuple[type, ...], methods: dict[str, Descriptor[Incomplete]]) -> Self: ...

class Strict(metaclass=MetaStrict): ...



from _typeshed import ConvertibleToInt, Incomplete, Unused
from typing import ClassVar, Literal, NoReturn, overload

from openpyxl.descriptors import Strict
from openpyxl.descriptors.base import Alias, Bool, Integer, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedInteger, NestedText
from openpyxl.descriptors.serialisable import Serialisable

from ..xml._functions_overloads import _HasTagAndGet

class NumFmt(Serialisable):
    formatCode: String[Literal[False]]
    sourceLinked: Bool[Literal[False]]
    def __init__(self, formatCode: str, sourceLinked: _ConvertibleToBool = False) -> None: ...

class NumberValueDescriptor(NestedText[Incomplete, Incomplete]):
    allow_none: bool
    expected_type: type[Incomplete]
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...  # type: ignore[override]

class NumVal(Serialisable):
    idx: Integer[Literal[False]]
    formatCode: NestedText[str, Literal[True]]
    v: Incomplete
    def __init__(self, idx: ConvertibleToInt, formatCode: object = None, v=None) -> None: ...

class NumData(Serialisable):
    formatCode: NestedText[str, Literal[True]]
    ptCount: NestedInteger[Literal[True]]
    pt: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        formatCode: object = None,
        ptCount: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        pt=(),
        extLst: Unused = None,
    ) -> None: ...

class NumRef(Serialisable):
    f: NestedText[str, Literal[False]]
    ref: Alias
    numCache: Typed[NumData, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, f: object = None, numCache: NumData | None = None, extLst: Unused = None) -> None: ...

class StrVal(Serialisable):
    tagname: ClassVar[str]
    idx: Integer[Literal[False]]
    v: NestedText[str, Literal[False]]
    def __init__(self, idx: ConvertibleToInt = 0, v: object = None) -> None: ...

class StrData(Serialisable):
    tagname: ClassVar[str]
    ptCount: NestedInteger[Literal[True]]
    pt: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self, ptCount: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None, pt=(), extLst: Unused = None
    ) -> None: ...

class StrRef(Serialisable):
    tagname: ClassVar[str]
    f: NestedText[str, Literal[True]]
    strCache: Typed[StrData, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, f: object = None, strCache: StrData | None = None, extLst: Unused = None) -> None: ...

class NumDataSource(Serialisable):
    numRef: Typed[NumRef, Literal[True]]
    numLit: Typed[NumData, Literal[True]]
    def __init__(self, numRef: NumRef | None = None, numLit: NumData | None = None) -> None: ...

class Level(Serialisable):
    tagname: ClassVar[str]
    pt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, pt=()) -> None: ...

class MultiLevelStrData(Serialisable):
    tagname: ClassVar[str]
    ptCount: Integer[Literal[True]]
    lvl: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, ptCount: ConvertibleToInt | None = None, lvl=(), extLst: Unused = None) -> None: ...

class MultiLevelStrRef(Serialisable):
    tagname: ClassVar[str]
    f: NestedText[str, Literal[False]]
    multiLvlStrCache: Typed[MultiLevelStrData, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, f: object = None, multiLvlStrCache: MultiLevelStrData | None = None, extLst: Unused = None) -> None: ...

class AxDataSource(Serialisable):
    tagname: ClassVar[str]
    numRef: Typed[NumRef, Literal[True]]
    numLit: Typed[NumData, Literal[True]]
    strRef: Typed[StrRef, Literal[True]]
    strLit: Typed[StrData, Literal[True]]
    multiLvlStrRef: Typed[MultiLevelStrRef, Literal[True]]
    @overload
    def __init__(
        self, numRef: None = None, numLit: None = None, strRef: None = None, strLit: None = None, multiLvlStrRef: None = None
    ) -> NoReturn: ...
    @overload
    def __init__(
        self,
        numRef: NumRef | None = None,
        numLit: NumData | None = None,
        strRef: StrRef | None = None,
        strLit: StrData | None = None,
        multiLvlStrRef: MultiLevelStrRef | None = None,
    ) -> None: ...



from _typeshed import Incomplete
from typing import Any
from typing_extensions import TypeAlias

from ..xml._functions_overloads import _lxml_Element, _ParentElement

_RootElement: TypeAlias = _ParentElement[Any] | _lxml_Element

vmlns: str
officens: str
excelns: str

class ShapeWriter:
    vml: Incomplete
    vml_path: Incomplete
    comments: Incomplete
    def __init__(self, comments) -> None: ...
    def add_comment_shapetype(self, root: _RootElement) -> None: ...
    def add_comment_shape(self, root: _RootElement, idx, coord, height, width) -> None: ...
    # Any object missing "findall" is replaced by an Element
    def write(self, root: _RootElement | None) -> str: ...



from zipfile import ZipFile

from openpyxl.chart._chart import ChartBase
from openpyxl.drawing.image import Image

def find_images(archive: ZipFile, path: str) -> tuple[list[ChartBase], list[Image]]: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.chart.axis import ChartLines, NumericAxis, SeriesAxis, TextAxis
from openpyxl.chart.label import DataLabelList
from openpyxl.descriptors.base import Alias, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedBool, NestedSet

from ..xml._functions_overloads import _HasTagAndGet
from ._chart import ChartBase

_AreaChartBaseGrouping: TypeAlias = Literal["percentStacked", "standard", "stacked"]

class _AreaChartBase(ChartBase):
    grouping: NestedSet[_AreaChartBaseGrouping]
    varyColors: NestedBool[Literal[True]]
    ser: Incomplete
    dLbls: Typed[DataLabelList, Literal[True]]
    dataLabels: Alias
    dropLines: Typed[ChartLines, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        grouping: _HasTagAndGet[_AreaChartBaseGrouping] | _AreaChartBaseGrouping = "standard",
        varyColors: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        ser=(),
        dLbls: DataLabelList | None = None,
        dropLines: ChartLines | None = None,
    ) -> None: ...

class AreaChart(_AreaChartBase):
    tagname: ClassVar[str]
    # Same as parent
    # grouping = _AreaChartBase.grouping
    # varyColors = _AreaChartBase.varyColors
    # ser = _AreaChartBase.ser
    # dLbls = _AreaChartBase.dLbls
    # dropLines = _AreaChartBase.dropLines
    x_axis: Typed[TextAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, axId: Unused = None, extLst: Unused = None, **kw) -> None: ...

class AreaChart3D(AreaChart):
    tagname: ClassVar[str]
    # Same as parent and grandparent
    # grouping = _AreaChartBase.grouping
    # varyColors = _AreaChartBase.varyColors
    # ser = _AreaChartBase.ser
    # dLbls = _AreaChartBase.dLbls
    # dropLines = _AreaChartBase.dropLines
    gapDepth: Incomplete
    x_axis: Typed[TextAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    z_axis: Typed[SeriesAxis, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, gapDepth=None, **kw) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, Literal

from openpyxl.descriptors.base import String
from openpyxl.descriptors.sequence import Sequence
from openpyxl.descriptors.serialisable import Serialisable

class Hyperlink(Serialisable):
    tagname: ClassVar[str]
    ref: String[Literal[False]]
    location: String[Literal[True]]
    tooltip: String[Literal[True]]
    display: String[Literal[True]]
    id: Incomplete
    target: String[Literal[True]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        ref: str,
        location: str | None = None,
        tooltip: str | None = None,
        display: str | None = None,
        id=None,
        target: str | None = None,
    ) -> None: ...

class HyperlinkList(Serialisable):
    tagname: ClassVar[str]
    hyperlink: Sequence[list[Hyperlink]]
    def __init__(self, hyperlink: list[Hyperlink] | tuple[Hyperlink, ...] = ()) -> None: ...



from _typeshed import SupportsGetItem
from collections.abc import Generator

from openpyxl import _VisibilityType
from openpyxl.cell import _CellGetValue, _CellOrMergedCell
from openpyxl.utils.cell import _RangeBoundariesTuple
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.worksheet import Worksheet

def read_dimension(source) -> _RangeBoundariesTuple | None: ...

class ReadOnlyWorksheet:
    cell = Worksheet.cell
    iter_rows = Worksheet.iter_rows
    # Same as Worksheet.values
    # https://github.com/python/mypy/issues/6700
    @property
    def values(self) -> Generator[tuple[_CellGetValue, ...], None, None]: ...
    # Same as Worksheet.rows
    # https://github.com/python/mypy/issues/6700
    @property
    def rows(self) -> Generator[tuple[_CellOrMergedCell, ...], None, None]: ...
    __getitem__ = Worksheet.__getitem__
    __iter__ = Worksheet.__iter__
    parent: Workbook
    title: str
    sheet_state: _VisibilityType
    def __init__(
        self, parent_workbook: Workbook, title: str, worksheet_path, shared_strings: SupportsGetItem[int, str]
    ) -> None: ...
    def calculate_dimension(self, force: bool = False): ...
    def reset_dimensions(self) -> None: ...
    @property
    def min_row(self) -> int: ...
    @property
    def max_row(self) -> int | None: ...
    @property
    def min_column(self) -> int: ...
    @property
    def max_column(self) -> int | None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal

from openpyxl.descriptors.base import Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.xml.functions import Element

class Record(Serialisable):
    tagname: ClassVar[str]
    m: Incomplete
    n: Incomplete
    b: Incomplete
    e: Incomplete
    s: Incomplete
    d: Incomplete
    x: Incomplete
    def __init__(self, _fields=(), m=None, n=None, b=None, e=None, s=None, d=None, x=None) -> None: ...

class RecordList(Serialisable):
    mime_type: str
    rel_type: str
    tagname: ClassVar[str]
    r: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(self, count: Unused = None, r=(), extLst: ExtensionList | None = None) -> None: ...
    @property
    def count(self) -> int: ...
    def to_tree(self) -> Element: ...  # type: ignore[override]
    @property
    def path(self) -> str: ...



from _typeshed import ConvertibleToInt, Incomplete, Unused
from typing import ClassVar, Literal, overload
from typing_extensions import TypeAlias

from openpyxl.descriptors import Float, Strict
from openpyxl.descriptors.base import Bool, Integer, NoneSet, Set, String, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.colors import Color, ColorDescriptor
from openpyxl.styles.differential import DifferentialStyle

_IconSetIconSet: TypeAlias = Literal[
    "3Arrows",
    "3ArrowsGray",
    "3Flags",
    "3TrafficLights1",
    "3TrafficLights2",
    "3Signs",
    "3Symbols",
    "3Symbols2",
    "4Arrows",
    "4ArrowsGray",
    "4RedToBlack",
    "4Rating",
    "4TrafficLights",
    "5Arrows",
    "5ArrowsGray",
    "5Rating",
    "5Quarters",
]
_RuleOperator: TypeAlias = Literal[
    "lessThan",
    "lessThanOrEqual",
    "equal",
    "notEqual",
    "greaterThanOrEqual",
    "greaterThan",
    "between",
    "notBetween",
    "containsText",
    "notContains",
    "beginsWith",
    "endsWith",
]
_RuleTimePeriod: TypeAlias = Literal[
    "today", "yesterday", "tomorrow", "last7Days", "thisMonth", "lastMonth", "nextMonth", "thisWeek", "lastWeek", "nextWeek"
]
_FormatObjectType: TypeAlias = Literal["num", "percent", "max", "min", "formula", "percentile"]
_RuleType: TypeAlias = Literal[
    "expression",
    "cellIs",
    "colorScale",
    "dataBar",
    "iconSet",
    "top10",
    "uniqueValues",
    "duplicateValues",
    "containsText",
    "notContainsText",
    "beginsWith",
    "endsWith",
    "containsBlanks",
    "notContainsBlanks",
    "containsErrors",
    "notContainsErrors",
    "timePeriod",
    "aboveAverage",
]

class ValueDescriptor(Float[Incomplete]):
    expected_type: type[Incomplete]
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...

class FormatObject(Serialisable):
    tagname: ClassVar[str]
    type: Set[_FormatObjectType]
    val: Incomplete
    gte: Bool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self, type: _FormatObjectType, val=None, gte: _ConvertibleToBool | None = None, extLst: Unused = None
    ) -> None: ...

class RuleType(Serialisable):
    cfvo: Incomplete

class IconSet(RuleType):
    tagname: ClassVar[str]
    iconSet: NoneSet[_IconSetIconSet]
    showValue: Bool[Literal[True]]
    percent: Bool[Literal[True]]
    reverse: Bool[Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    cfvo: Incomplete
    def __init__(
        self,
        iconSet: _IconSetIconSet | Literal["none"] | None = None,
        showValue: _ConvertibleToBool | None = None,
        percent: _ConvertibleToBool | None = None,
        reverse: _ConvertibleToBool | None = None,
        cfvo=None,
    ) -> None: ...

class DataBar(RuleType):
    tagname: ClassVar[str]
    minLength: Integer[Literal[True]]
    maxLength: Integer[Literal[True]]
    showValue: Bool[Literal[True]]
    color: ColorDescriptor[Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    cfvo: Incomplete
    @overload
    def __init__(
        self,
        minLength: ConvertibleToInt | None = None,
        maxLength: ConvertibleToInt | None = None,
        showValue: _ConvertibleToBool | None = None,
        cfvo=None,
        *,
        color: str | Color,
    ) -> None: ...
    @overload
    def __init__(
        self,
        minLength: ConvertibleToInt | None,
        maxLength: ConvertibleToInt | None,
        showValue: _ConvertibleToBool | None,
        cfvo: Incomplete | None,
        color: str | Color,
    ) -> None: ...

class ColorScale(RuleType):
    tagname: ClassVar[str]
    color: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    cfvo: Incomplete
    def __init__(self, cfvo=None, color=None) -> None: ...

class Rule(Serialisable):
    tagname: ClassVar[str]
    type: Set[_RuleType]
    dxfId: Integer[Literal[True]]
    priority: Integer[Literal[False]]
    stopIfTrue: Bool[Literal[True]]
    aboveAverage: Bool[Literal[True]]
    percent: Bool[Literal[True]]
    bottom: Bool[Literal[True]]
    operator: NoneSet[_RuleOperator]
    text: String[Literal[True]]
    timePeriod: NoneSet[_RuleTimePeriod]
    rank: Integer[Literal[True]]
    stdDev: Integer[Literal[True]]
    equalAverage: Bool[Literal[True]]
    formula: Incomplete
    colorScale: Typed[ColorScale, Literal[True]]
    dataBar: Typed[DataBar, Literal[True]]
    iconSet: Typed[IconSet, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    dxf: Typed[DifferentialStyle, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        type: _RuleType,
        dxfId: ConvertibleToInt | None = None,
        priority: ConvertibleToInt = 0,
        stopIfTrue: _ConvertibleToBool | None = None,
        aboveAverage: _ConvertibleToBool | None = None,
        percent: _ConvertibleToBool | None = None,
        bottom: _ConvertibleToBool | None = None,
        operator: _RuleOperator | Literal["none"] | None = None,
        text: str | None = None,
        timePeriod: _RuleTimePeriod | Literal["none"] | None = None,
        rank: ConvertibleToInt | None = None,
        stdDev: ConvertibleToInt | None = None,
        equalAverage: _ConvertibleToBool | None = None,
        formula=(),
        colorScale: ColorScale | None = None,
        dataBar: DataBar | None = None,
        iconSet: IconSet | None = None,
        extLst: Unused = None,
        dxf: DifferentialStyle | None = None,
    ) -> None: ...

def ColorScaleRule(
    start_type=None,
    start_value=None,
    start_color=None,
    mid_type=None,
    mid_value=None,
    mid_color=None,
    end_type=None,
    end_value=None,
    end_color=None,
): ...
def FormulaRule(formula=None, stopIfTrue=None, font=None, border=None, fill=None): ...
def CellIsRule(operator=None, formula=None, stopIfTrue=None, font=None, border=None, fill=None): ...
def IconSetRule(icon_style=None, type=None, values=None, showValue=None, percent=None, reverse=None): ...
def DataBarRule(
    start_type=None, start_value=None, end_type=None, end_value=None, color=None, showValue=None, minLength=None, maxLength=None
): ...



from .area_chart import AreaChart as AreaChart, AreaChart3D as AreaChart3D
from .bar_chart import BarChart as BarChart, BarChart3D as BarChart3D
from .bubble_chart import BubbleChart as BubbleChart
from .line_chart import LineChart as LineChart, LineChart3D as LineChart3D
from .pie_chart import (
    DoughnutChart as DoughnutChart,
    PieChart as PieChart,
    PieChart3D as PieChart3D,
    ProjectedPieChart as ProjectedPieChart,
)
from .radar_chart import RadarChart as RadarChart
from .reference import Reference as Reference
from .scatter_chart import ScatterChart as ScatterChart
from .stock_chart import StockChart as StockChart
from .surface_chart import SurfaceChart as SurfaceChart, SurfaceChart3D as SurfaceChart3D



from _typeshed import ConvertibleToFloat
from collections.abc import Iterator
from typing import ClassVar, Final, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Alias, Bool, Min, MinMax, NoneSet, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

_HorizontalAlignmentsType: TypeAlias = Literal[
    "general", "left", "center", "right", "fill", "justify", "centerContinuous", "distributed"
]
_VerticalAlignmentsType: TypeAlias = Literal["top", "center", "bottom", "justify", "distributed"]

horizontal_alignments: Final[tuple[_HorizontalAlignmentsType, ...]]
vertical_aligments: Final[tuple[_VerticalAlignmentsType, ...]]

class Alignment(Serialisable):
    tagname: ClassVar[str]
    horizontal: NoneSet[_HorizontalAlignmentsType]
    vertical: NoneSet[_VerticalAlignmentsType]
    textRotation: NoneSet[int]
    text_rotation: Alias
    wrapText: Bool[Literal[True]]
    wrap_text: Alias
    shrinkToFit: Bool[Literal[True]]
    shrink_to_fit: Alias
    indent: MinMax[float, Literal[False]]
    relativeIndent: MinMax[float, Literal[False]]
    justifyLastLine: Bool[Literal[True]]
    readingOrder: Min[float, Literal[False]]
    def __init__(
        self,
        horizontal=None,
        vertical=None,
        textRotation: int = 0,
        wrapText: _ConvertibleToBool | None = None,
        shrinkToFit: _ConvertibleToBool | None = None,
        indent: ConvertibleToFloat = 0,
        relativeIndent: ConvertibleToFloat = 0,
        justifyLastLine: _ConvertibleToBool | None = None,
        readingOrder: ConvertibleToFloat = 0,
        text_rotation=None,
        wrap_text=None,
        shrink_to_fit=None,
        mergeCell=None,
    ) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

from openpyxl.descriptors.base import Alias, Integer, NoneSet, Typed, _ConvertibleToBool
from openpyxl.descriptors.nested import NestedString, NestedText, _NestedNoneSetParam
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.colors import Color
from openpyxl.styles.fonts import Font, _FontScheme, _FontU, _FontVertAlign

from ..xml._functions_overloads import _HasTagAndGet

_PhoneticPropertiesType: TypeAlias = Literal["halfwidthKatakana", "fullwidthKatakana", "Hiragana", "noConversion"]
_PhoneticPropertiesAlignment: TypeAlias = Literal["noControl", "left", "center", "distributed"]

class PhoneticProperties(Serialisable):
    tagname: ClassVar[str]
    fontId: Integer[Literal[False]]
    type: NoneSet[_PhoneticPropertiesType]
    alignment: NoneSet[_PhoneticPropertiesAlignment]
    def __init__(
        self,
        fontId: ConvertibleToInt,
        type: _PhoneticPropertiesType | Literal["none"] | None = None,
        alignment: _PhoneticPropertiesAlignment | Literal["none"] | None = None,
    ) -> None: ...

_PhoneticProperties: TypeAlias = PhoneticProperties

class PhoneticText(Serialisable):
    tagname: ClassVar[str]
    sb: Integer[Literal[False]]
    eb: Integer[Literal[False]]
    t: NestedText[str, Literal[False]]
    text: Alias
    def __init__(self, sb: ConvertibleToInt, eb: ConvertibleToInt, t: object = None) -> None: ...

class InlineFont(Font):
    tagname: ClassVar[str]
    rFont: NestedString[Literal[True]]
    # Same as parent
    # charset = Font.charset
    # family = Font.family
    # b = Font.b
    # i = Font.i
    # strike = Font.strike
    # outline = Font.outline
    # shadow = Font.shadow
    # condense = Font.condense
    # extend = Font.extend
    # color = Font.color
    # sz = Font.sz
    # u = Font.u
    # vertAlign = Font.vertAlign
    # scheme = Font.scheme
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        rFont: object = None,
        charset: _HasTagAndGet[ConvertibleToInt | None] | ConvertibleToInt | None = None,
        family: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        b: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool = None,
        i: _HasTagAndGet[_ConvertibleToBool] | _ConvertibleToBool = None,
        strike: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        outline: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        shadow: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        condense: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        extend: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        color: Color | None = None,
        sz: _HasTagAndGet[ConvertibleToFloat | None] | ConvertibleToFloat | None = None,
        u: _NestedNoneSetParam[_FontU] = None,
        vertAlign: _NestedNoneSetParam[_FontVertAlign] = None,
        scheme: _NestedNoneSetParam[_FontScheme] = None,
    ) -> None: ...

class RichText(Serialisable):
    tagname: ClassVar[str]
    rPr: Typed[InlineFont, Literal[True]]
    font: Alias
    t: NestedText[str, Literal[True]]
    text: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, rPr: InlineFont | None = None, t: object = None) -> None: ...

class Text(Serialisable):
    tagname: ClassVar[str]
    t: NestedText[str, Literal[True]]
    plain: Alias
    r: Incomplete
    formatted: Alias
    rPh: Incomplete
    phonetic: Alias
    phoneticPr: Typed[_PhoneticProperties, Literal[True]]
    PhoneticProperties: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, t: object = None, r=(), rPh=(), phoneticPr: _PhoneticProperties | None = None) -> None: ...
    @property
    def content(self) -> str: ...
